
// Titre: Util.cpp
// Auteur: Pascal Mignot - Universit de Reims
// Cration : 28 novembre 2003

#include "Util.h"
#include <assert.h>

// pour le timer haute-resolution (calcul du FPS)
double    tstart, tstop, tcorr, tfreq;

// pour affichage sur l'ecran
ID3DXFont*   pFont = NULL; 

// variable qui contient timeGetTime()  chaque render
UINT         ctime;
int			 nframes = 0;

////////////////////////////////////////////////////////
// pour le calcul du FPS en haute resolution
////////////////////////////////////////////////////////
double GetFrequency(void) { 
    LARGE_INTEGER timer;
    QueryPerformanceFrequency(&timer); 
    return (double)timer.QuadPart;
}

double GetTimer(void) {
    LARGE_INTEGER timer;
    QueryPerformanceCounter(&timer);   
    return (double)timer.QuadPart;
}

void InitTimer(void) { 
    tfreq = GetFrequency(); 
    tstart = GetTimer(); 
    tstop  = GetTimer(); 
    tcorr  = tstop-tstart;
	nframes = 0;
}

////////////////////////////////////////////////////////
// pour l'affichage de texte a l'ecran
////////////////////////////////////////////////////////
HRESULT InitFont(void) {
    HRESULT h;
    HDC hDC = GetDC( NULL );
    int nHeight = -MulDiv( 9, GetDeviceCaps(hDC, LOGPIXELSY), 72 );
    ReleaseDC( NULL, hDC );
    h = D3DXCreateFont( g_pd3dDevice, nHeight, 0, FW_BOLD, 0, FALSE, DEFAULT_CHARSET, 
        OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
        TEXT("Arial"), &pFont );
    return h;
}

void ReleaseFont(void) {
    SAFE_RELEASE(pFont);
}

/////////////////////////////////////////////////////////////////////////
// affichage du FPS
// [in] nline : le numero de la ligne sur laquelle l'affichage est fait
/////////////////////////////////////////////////////////////////////////
#define NREFRESH  500  // nombre de millisecondes entre 2 rafraichissements du FPS
void drawFPS(int nline, D3DCOLOR color) {
    static UINT  ltime=0;
    //static int   nframes=0;

    if( pFont == NULL ) return;
    // variable pour le timing
    static char str[100]="FPS : waiting ...";

    // calcul du FPS
    nframes++;

    if (ctime - ltime > NREFRESH) {
        tstop = GetTimer();
        double cur_fps = nframes / ( (tstop - tstart - tcorr) / tfreq );
        if (ltime>0) sprintf(str,"FPS: %.2lf\n",cur_fps);
        nframes = 0;
        ltime = ctime;
        tstart = GetTimer();		
    }

    // affichage du FPS
    RECT rc;
    SetRect( &rc, 2, 15*nline, 0, 0 );
    pFont->DrawText( NULL, str, -1, &rc, DT_NOCLIP, color);
}

/////////////////////////////////////////////////////////////////////////
// affichage d'une chaine de caractere
// [in] nline : le numero de la ligne sur laquelle l'affichage est fait
// [in] str : chaine a afficher
/////////////////////////////////////////////////////////////////////////
void drawText( int nline, char *str, D3DCOLOR color ) {
    if( pFont == NULL ) return;
    RECT rc;
    SetRect( &rc, 2, 15*nline, 0, 0 );
    pFont->DrawText( NULL, str, -1, &rc, DT_NOCLIP, color);    
}